package POWER::Email;

use version; $VERSION = qv('0.0.1');

use warnings;
use strict;
use Carp;

use MIME::Base64;

use Perl6::Export::Attrs;

use POWER::Utils;


my $Mail_Prog = '/usr/sbin/sendmail -t';
sub Email :Export(:DEFAULT) {
    my ($file, $hash_ref, $attach) = @_;
    local *MAIL;
    open(MAIL,"| $Mail_Prog") or croak "Email: $!";
    local $_ = $file =~ /\r|\n/ ? $file : Cat($file);
    s/@~(.*?)~@/$hash_ref->{$1}/gs;
    if (keys %$attach) {
        my ($head, $body) = split /\r?\n\r?\n/,$_,2;
	$head .= "\n";
	my $chead; $chead .= $1 while $head =~ s/^(Content-.*\r?\n)//im;
	$chead .= "Content-Type: TEXT/PLAIN; charset=US-ASCII\n"
	    if $chead !~ /^Content-Type:/mi;
        my $bound = RandCh(35);
        $head .= "MIME-Version: 1.0\n".
            "Content-Type: MULTIPART/MIXED; BOUNDARY=\"$bound\"\n\n";
        $body = "--$bound\n".$chead."\n".$body."\n";
        for $file (keys %$attach) {
            $attach->{$file} = Cat($file) unless defined $attach->{$file};
            (my $filename = $file) =~ s!.*/!!;
            $body .= "--$bound\n".
                "Content-Type: APPLICATION/octet-stream; name=\"$filename\"\n".
                "Content-Transfer-Encoding: BASE64\n".
                "Content-Disposition: attachment; filename=\"$filename\"\n\n".
                encode_base64($attach->{$file});
        }
        $_ = $head . $body . "--$bound--\n";
    }
    print MAIL $_;
}
    

1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Email - Send email using templates.


=head1 VERSION

This document describes POWER::Email version 0.0.1


=head1 SYNOPSIS

 use POWER::Email;

 Email($template, \%fields);

=head1 DESCRIPTION

Fill template (from file or scalar) with given values and send email.
Attaches supported. PGP will be supported shortly.


=head1 EXPORTS

=over

=item :DEFAULT

Email()


=back

=head1 INTERFACE 

=over

=item B<Email>( $template, $data [, $attach] )

Send email.

If $template looks like a file name than email template will be read from it.
In other case scalar $template will be treated as template text.

$data must be a hash reference, and this hash will be used to substitute
fields in email template.
Fields format is: @~field_name~@ .

$attach must be a hash reference. Keys in this hash treated as file names.
If %$attach element has defined value this value used as file content, else
file content is read from disk.

If $attach is set, then template will be automatically reformatted to
MULTIPART/MIXED format. First part will be template body, second and other
parts will be attached files. All /^Content-/ headers found in template
will be moved to first part's headers. If there will be no /^Content-Type:/
header in template then it will be set to "TEXT/PLAIN; charset=US-ASCII".

After template is processed it is sent to mail program.
Mail program usually is "/usr/bin/sendmail -t".

Example:

 $template = <<EOT;
 From: Email robot <nobody>
 To: @~email~@
 Subject: @~subj~@
 Content-Type: text/plain; charset=Windows-1251
 Content-Transfer-Encoding: 8bit

 Dear @~name~@.
 This message is a test.
 
 Sincerely, Robot.
 EOT
 $data = {
    email => 'user@remote.host.com',
    subj  => "Test",
    name  => "John Doe",
    };
 $attach = {
    "outbound/pic/1.jpg" => undef,
    "comment.txt" => "1.jpg is my photo.\n\nJonh.",
    };
 Email($template, $data, $attach);


=back

=head1 DIAGNOSTICS

=over

=item C<< Email: $! >>

Email() try to open pipe to '/usr/sbin/sendmail -t' and fail.


=back


=head1 CONFIGURATION AND ENVIRONMENT

POWER::Email requires no configuration files or environment variables.


=head1 DEPENDENCIES

=over

=item version

=item Perl6::Export::Attrs

=item POWER::Utils


=back

=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-email@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005, Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
